%--------------------------------------------------------------------------
% Simulation Experiment for GMM Averaging Estimator 
%--------------------------------------------------------------------------
% Generate simulation data sets
% Data structure: (Yi,Xi,Zi's)
%--------------------------------------------------------------------------
function f = data(theta,beta,delta,n,model)
%--------------------------------------------------------------------------
% theta - parameter of interest in the main regression
% beta  - parameter in X regression
% delta - weight in the invalid IVs
% n     - sample size
% model - different DGPs 
%       = 1, the DGP S1 in the paper
%       = 2, the DGP S4 in Appendix H (error is t(2))
%       = 3, the DGP S2 in the paper
%       = 4, the DGP S3 in the paper
%--------------------------------------------------------------------------
if model == 1
    %----------------------------------------------------------------------
    Sigma = [eye(18) zeros(18,7); zeros(6,18) eye(6) .25*ones(6,1); zeros(1,18) .25*ones(1,6) 1];
    %----------------------------------------------------------------------
    Z     = mvnrnd(zeros(25,1),Sigma,n);
    
    eta   = exprnd(1,[n,1]) - 1;
    
    Z(:,25) = (Z(:,25) + eta)/2;
    %----------------------------------------------------------------------
    Data         = zeros(n,25);
    Data(:,8:19) = Z(:,1:12);
    Data(:,20)   = Z(:,13)*(1 - delta(1)^2)^.5 + delta(1)*(Z(:,19) + Z(:,25));
    Data(:,21)   = Z(:,14)*(1 - delta(2)^2)^.5 + delta(2)*(Z(:,20) + Z(:,25));
    Data(:,22)   = Z(:,15)*(1 - delta(3)^2)^.5 + delta(3)*(Z(:,21) + Z(:,25));
    Data(:,23)   = Z(:,16)*(1 - delta(4)^2)^.5 + delta(4)*(Z(:,22) + Z(:,25));
    Data(:,24)   = Z(:,17)*(1 - delta(5)^2)^.5 + delta(5)*(Z(:,23) + Z(:,25));
    Data(:,25)   = Z(:,18)*(1 - delta(6)^2)^.5 + delta(6)*(Z(:,24) + Z(:,25));
    Data(:,2)    = (Z(:,1)  + Z(:,2)) *beta + Z(:,13) + Z(:,19);
    Data(:,3)    = (Z(:,3)  + Z(:,4)) *beta + Z(:,14) + Z(:,20);
    Data(:,4)    = (Z(:,5)  + Z(:,6)) *beta + Z(:,15) + Z(:,21);
    Data(:,5)    = (Z(:,7)  + Z(:,8)) *beta + Z(:,16) + Z(:,22);
    Data(:,6)    = (Z(:,9)  + Z(:,10))*beta + Z(:,17) + Z(:,23);
    Data(:,7)    = (Z(:,11) + Z(:,12))*beta + Z(:,18) + Z(:,24);
    Data(:,1)    = Data(:,2:7)*theta + Z(:,25);
    %---------------------------------------------------------------------- 
elseif model == 2
    %----------------------------------------------------------------------
    Sigma = [eye(18) zeros(18,7); zeros(6,18) eye(6) .25*ones(6,1); zeros(1,18) .25*ones(1,6) 1];
    %----------------------------------------------------------------------
    Z     = mvnrnd(zeros(25,1),Sigma,n); df = 2;
    
    U     = mvnrnd(zeros(df,1),eye(df),n);
    
    Z(:,25) = Z(:,25)./((U.^2)*ones(df,1)/df).^.5;
    %----------------------------------------------------------------------
    Data         = zeros(n,25);
    Data(:,8:19) = Z(:,1:12);
    Data(:,20)   = Z(:,13)*(1 - delta(1)^2)^.5 + delta(1)*(Z(:,19));
    Data(:,21)   = Z(:,14)*(1 - delta(2)^2)^.5 + delta(2)*(Z(:,20));
    Data(:,22)   = Z(:,15)*(1 - delta(3)^2)^.5 + delta(3)*(Z(:,21));
    Data(:,23)   = Z(:,16)*(1 - delta(4)^2)^.5 + delta(4)*(Z(:,22));
    Data(:,24)   = Z(:,17)*(1 - delta(5)^2)^.5 + delta(5)*(Z(:,23));
    Data(:,25)   = Z(:,18)*(1 - delta(6)^2)^.5 + delta(6)*(Z(:,24));
    Data(:,2)    = (Z(:,1)  + Z(:,2)) *beta + Z(:,13) + Z(:,19);
    Data(:,3)    = (Z(:,3)  + Z(:,4)) *beta + Z(:,14) + Z(:,20);
    Data(:,4)    = (Z(:,5)  + Z(:,6)) *beta + Z(:,15) + Z(:,21);
    Data(:,5)    = (Z(:,7)  + Z(:,8)) *beta + Z(:,16) + Z(:,22);
    Data(:,6)    = (Z(:,9)  + Z(:,10))*beta + Z(:,17) + Z(:,23);
    Data(:,7)    = (Z(:,11) + Z(:,12))*beta + Z(:,18) + Z(:,24);
    Data(:,1)    = Data(:,2:7)*theta + Z(:,25);
    %---------------------------------------------------------------------- 
elseif model == 3    
    %----------------------------------------------------------------------
    Sigma = [eye(18) zeros(18,7); zeros(6,18) eye(6) .25*ones(6,1); zeros(1,18) .25*ones(1,6) 1];
    %----------------------------------------------------------------------
    Z     = mvnrnd(zeros(25,1),Sigma,n);
    %----------------------------------------------------------------------
    Data         = zeros(n,25);
    Data(:,8:19) = Z(:,1:12);
    Data(:,20)   = Z(:,13)*(1 - delta(1)^2)^.5 + delta(1)*(Z(:,19) + Z(:,25));
    Data(:,21)   = Z(:,14)*(1 - delta(2)^2)^.5 + delta(2)*(Z(:,20) + Z(:,25));
    Data(:,22)   = Z(:,15)*(1 - delta(3)^2)^.5 + delta(3)*(Z(:,21) + Z(:,25));
    Data(:,23)   = Z(:,16)*(1 - delta(4)^2)^.5 + delta(4)*(Z(:,22) + Z(:,25));
    Data(:,24)   = Z(:,17)*(1 - delta(5)^2)^.5 + delta(5)*(Z(:,23) + Z(:,25));
    Data(:,25)   = Z(:,18)*(1 - delta(6)^2)^.5 + delta(6)*(Z(:,24) + Z(:,25));
    Data(:,2)    = (Z(:,1)  + Z(:,2)) *beta + Z(:,13) + Z(:,19);
    Data(:,3)    = (Z(:,3)  + Z(:,4)) *beta + Z(:,14) + Z(:,20);
    Data(:,4)    = (Z(:,5)  + Z(:,6)) *beta + Z(:,15) + Z(:,21);
    Data(:,5)    = (Z(:,7)  + Z(:,8)) *beta + Z(:,16) + Z(:,22);
    Data(:,6)    = (Z(:,9)  + Z(:,10))*beta + Z(:,17) + Z(:,23);
    Data(:,7)    = (Z(:,11) + Z(:,12))*beta + Z(:,18) + Z(:,24);
    Data(:,1)    = Data(:,2:7)*theta + Z(:,25);
    %----------------------------------------------------------------------  
elseif model == 4    
    %----------------------------------------------------------------------
    Sigma = [eye(13) zeros(13,6); zeros(5,13) eye(5) .25*ones(5,1); zeros(1,13) .25*ones(1,5) 1];
    %----------------------------------------------------------------------
    Z     = mvnrnd(zeros(19,1),Sigma,n);
    %----------------------------------------------------------------------
    Data=zeros(n,15);
    Data(:,2) = (Z(:,1)+Z(:,2)+Z(:,9))/sqrt(3);
    Data(:,3) = (Z(:,2)+Z(:,3)+Z(:,10))/sqrt(3);
    Data(:,4) = (Z(:,3)+Z(:,4)+Z(:,11))/sqrt(3);
    Data(:,5) = (Z(:,4)+Z(:,5)+Z(:,12))/sqrt(3);
    Data(:,6) = (Z(:,5)+Z(:,1)+Z(:,13))/sqrt(3);
    Data(:,8:10) = Z(:,6:8);
    Data(:,11) = Z(:,9)*(1 - delta(1)^2)^.5  + delta(1)*(Z(:,14) + Z(:,19));  
    Data(:,12) = Z(:,10)*(1 - delta(2)^2)^.5 + delta(2)*(Z(:,15) + Z(:,19));  
    Data(:,13) = Z(:,11)*(1 - delta(3)^2)^.5 + delta(3)*(Z(:,16) + Z(:,19));  
    Data(:,14) = Z(:,12)*(1 - delta(4)^2)^.5 + delta(4)*(Z(:,17) + Z(:,19));  
    Data(:,15) = Z(:,13)*(1 - delta(5)^2)^.5 + delta(5)*(Z(:,18) + Z(:,19));  
    Data(:,7)  = (Z(:,6) + Z(:,7) + Z(:,8))*beta + Z(:,9:18)*ones(10,1)/sqrt(10);
    Data(:,1)  = Data(:,2:7)*theta + Z(:,19);
    %----------------------------------------------------------------------
end

f = Data;

end

