%--------------------------------------------------------------------------
% Simulation Experiment for GMM Averaging Estimator
%--------------------------------------------------------------------------
% GMM estimator of linear regression model has explicit solution
% Data structure: (Yi,Xi,Zi's)
%--------------------------------------------------------------------------
function [paraest, fval, Weit, Sigma] = gmm4(Data,type)
%--------------------------------------------------------------------------
% paraest - GMM estimate
% fval    - GMM criterion value
% Weit    - efficient weight matrix
% Sigma   - estimator of variance-covariance matrix of the GMM estimator
% Data    - data set
% type    - type of estimator
%     = 1 - conservative GMM
%     = 2 - aggressive GMM 
%--------------------------------------------------------------------------
% Sample size and model number
%--------------------------------------------------------------------------
n = length(Data); Y = Data(:,1); X = Data(:,2:7);       

Zc = [Data(:,2:6), Data(:,8:10)]; Zm = Data(:,11:15); cor = 8; mis = 5;
%--------------------------------------------------------------------------
% Define conservative GMM
%--------------------------------------------------------------------------
if type == 1
    G       = -Zc'*X/n;
    paraest = pinv(X'*Zc*pinv(Zc'*Zc)*Zc'*X)*(X'*Zc*pinv(Zc'*Zc)*Zc'*Y);
    U       = Y - X*paraest;
    m       = (U*ones(1,cor)).*Zc;
    mbar    = mean(m);
    Weit    = pinv(m'*m/n - mbar'*mbar); 
    fval    = n*mbar*Weit*mbar'; 
    Sigma   = pinv(G'*Weit*G); 
%--------------------------------------------------------------------------
% Define aggressive GMM
%--------------------------------------------------------------------------
elseif type == 2
    Z       = [Zc Zm]; 
    G       = -Z'*X/n;
    para0   = pinv(X'*Zc*pinv(Zc'*Zc)*Zc'*X)*(X'*Zc*pinv(Zc'*Zc)*Zc'*Y);
    U0      = Y - X*para0;
    m0      = (U0*ones(1,cor + mis)).*Z;
    mbar0   = mean(m0);
    Weit    = pinv(m0'*m0/n-mbar0'*mbar0);  
    paraest = pinv(X'*Z*Weit*Z'*X)*(X'*Z*Weit*Z'*Y);
    U       = Y - X*paraest; 
    m       = (U*ones(1,cor + mis)).*Z;
    mbar    = mean(m);
    fval    = n*mbar*Weit*mbar'; 
    Sigma   = pinv(G'*Weit*G); 
end

end